/*a simple calculator for complex number system
 *Akira Saitoh (1999,2002)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "Clicked.h"
#include "OtherFuncs.h"
#include "Copy.h"
#include "parenthesis.h"
#include "WriteOutput.h"

#define OneThird 3.33333333333333333333333333333333333\
33333333333333333333333333333333333333333e-1
/*
 *The size of the buffers are defined in 'cgtkcalc.h'.
 */
static char lastStr[CMD_SIZE] = "=";

void l_print_str (const char *str);
#define print_error() l_print_str("ERROR  ")
static void clear_last_command ();
static void mv_cursor_real ();
static void handle_const (double c, char *str);

/*Display a string on labels.*/
void l_print_str (const char *str)
{
/*If `lastStr' is "u_op", labels will be cleared when the next is input.
 *So let `laseStr' to be "u_op" here.*/
   strcpy (lastStr, "u_op");
   PrepareChange();
     gtk_label_set (GTK_LABEL (labelRe), str);
     gtk_label_set (GTK_LABEL (labelIm), str);
     gtk_label_set (GTK_LABEL (labelAbs), str);
     gtk_label_set (GTK_LABEL (labelArg), str);
}

/*Clear lastStr if it is a command.*/
static void clear_last_command ()
{
   if (is_Command (lastStr))
   {
      PrepareChange ();
      cz.r = 0.0; cz.i = 0.0;
      cz.abs = 0.0; cz.arg = 0.0;
      lastStr[0] = '\0';
      RenewLabels();  
      if (strcmp (prevCmd, "=") == 0)
      {   
         gtk_label_set (GTK_LABEL (labelOp), NULL);
         prevCmd[0] = '\0';
      }
   }
}

/*Move cursor to the real part after a calculation.*/
static void mv_cursor_real ()
{
   inputpos = REAL;
   /*toggle `real' button*/ 
   gtk_button_pressed (GTK_BUTTON (buttonList[NB_REAL].widget));
   gtk_button_released (GTK_BUTTON (buttonList[NB_REAL].widget));
}

/*This function is called in HandleDigit() to handle "pi" and "e".*/
static void handle_const (double c, char *str)
{
  switch (inputpos)
  {
    case REAL:
       cz.r = c; absarg (&cz);
       RenewRe (); RenewAbs (); RenewArg ();
       break;
    case IMAG:
       cz.i = c; absarg (&cz);
       RenewIm (); RenewAbs (); RenewArg ();
       break;
    case ABSO:
       cz.abs = c; realimag (&cz);
       RenewRe (); RenewIm (); RenewAbs ();
       break;
    case ARGU:
       cz.arg = c; realimag (&cz);
       RenewRe (); RenewIm (); RenewArg ();
  }
  strncpy (lastStr, str, 3);
}

/******************
    Handle Digit
 ******************/
void HandleDigit (GtkWidget *widget, gpointer data)
{  
   char buffer0[BUF_SIZE];
   char buffer1[CMD_SIZE];
   char bufsign[2] = " ";
   char *ltext;
   int is_sign = 0;
   int MaxSize;
 /*the maximum number of characters that can be on a label.*/
  MaxSize = BUF_SIZE - CMD_SIZE -1;

 /*Clear if lastStr is a command.*/
  clear_last_command ();
 /*Get label text of currently active part.*/
  switch (inputpos)
  {
    case REAL:
       gtk_label_get (GTK_LABEL (labelRe), &ltext); break;
    case IMAG:
       gtk_label_get (GTK_LABEL (labelIm), &ltext); break;
    case ABSO:
       gtk_label_get (GTK_LABEL (labelAbs), &ltext); break;
    case ARGU:
       gtk_label_get (GTK_LABEL (labelArg), &ltext);
  }
  if ((int)strlen(ltext) > MaxSize) return;
  strncpy_terminate (buffer0, ltext, MaxSize);
  strcpy (buffer1, (char *) data);

  if (*buffer0 == ' ') g_strchug (buffer0);

  if (*buffer1 == '.')
  {
    if (where_c_in_str (buffer0, '.')) return;
    if (where_c_in_str (buffer0, 'E')) return;
  }
  else if (strcmp (buffer1, "EE") == 0)
  {
    if (where_c_in_str (buffer0, 'E') == 0)
    {
      if (*buffer0 == 'E') return;
      strcpy (buffer1, "E+");
    } else return;
  }
  else if (strcmp (buffer1, "+/-") == 0)
  {
    int p;
    /*when there is "E+" in the Label*/
    if ((p = where_c_in_str (buffer0, 'E')))
    {
      p++;
      if (*(buffer0 + p) == '+')
         *(buffer0 + p) = '-';
      else
         *(buffer0 + p) = '+';
      strcpy (buffer1, " ");
    }
    else
    {
      strcpy (bufsign, "-"); strcpy (buffer1, " "); is_sign = 1;
    }
  }
  /*Note: "e" or "pi" is not a operation, but a value.
   *I think this style is valid as we handle complex numbers.*/
  else if (strcmp (buffer1, "e") == 0)
  {
    #define Num_e  2.7182818284590452353602874713526624977572\
47093699959574966967627724076630354
    handle_const (Num_e, buffer1);
    #undef Num_e
    return;
  }
  else if (strcmp (buffer1, "pi") == 0)
  {
    #define Num_pi  3.1415926535897932384626433832795028841971\
693993751058209749445923078164062862
    handle_const (Num_pi, buffer1);
    #undef Num_pi
    return;
  }

  strcpy (lastStr, buffer1);

 switch (inputpos)
 {
   case REAL:
   /*When input == "+/-" and label == "0"*/
      if(strcmp(buffer0, "0") == 0 && *buffer1 == ' ')
         buffer0[0] = '\0';
      str_plus (buffer0, buffer1);
      TrimLeadingZeros (buffer0);
      plus_str (bufsign, buffer0);
      if (is_sign) TrimLeadingSigns (buffer0);
      else      g_strchomp (buffer0);
      cz.r = atof (buffer0);
      absarg (&cz);
      RenewAbs(); RenewArg();
      gtk_label_set (GTK_LABEL (labelRe), buffer0);
      return;
   case IMAG:
   /*When input == "+/-" and label == "0"*/
      if(strcmp(buffer0, "0") == 0 && *buffer1 == ' ')
         buffer0[0] = '\0';
      str_plus (buffer0, buffer1);
      TrimLeadingZeros (buffer0);
      plus_str (bufsign, buffer0);
      if (is_sign) TrimLeadingSigns (buffer0);
      else      g_strchomp (buffer0);
      cz.i = atof (buffer0);
      absarg (&cz);
      RenewAbs(); RenewArg();
      gtk_label_set (GTK_LABEL (labelIm), buffer0);
      return;
   case ABSO:
      str_plus (buffer0, buffer1);
      TrimLeadingZeros (buffer0);
      g_strchomp (buffer0);
      cz.abs = atof (buffer0);
      realimag (&cz);
      RenewRe(); RenewIm();
      gtk_label_set (GTK_LABEL (labelAbs), buffer0);
      return;
   case ARGU:
      /*Because `buffer0' includes " rad " or " deg ", it must be trimed. */
      {
         int len0;
         if ((len0 = strlen (buffer0)) > 4)
         {
            char bs[BUF_SIZE];
            strncpy_terminate (bs, buffer0, len0 - 5);
            strcpy (buffer0, bs);
         }
      }
   /*When input == "+/-" and label == "0"*/
      if(strcmp(buffer0, "0") == 0 && *buffer1 == ' ')
         buffer0[0] = '\0';
      str_plus (buffer0, buffer1);
      TrimLeadingZeros (buffer0);
      plus_str (bufsign, buffer0);
      if (is_sign) TrimLeadingSigns (buffer0);
      else      g_strchomp (buffer0);
 
      switch (arg_unit){
        case RADIAN: cz.arg = atof (buffer0); break;
      /*translate PIrad into rad*/
        case PIRADIAN: cz.arg = tPI * atof (buffer0); break;
      /*translate deg into rad*/
        case DEGREE: cz.arg = (tPI / 180.0 * atof (buffer0)); break;
      }
       realimag (&cz);
       RenewRe(); RenewIm();
      switch (arg_unit){
        case RADIAN: str_plus (buffer0, " rad "); break;
        case PIRADIAN: str_plus (buffer0, "PIrad"); break;
        case DEGREE: str_plus (buffer0, " deg "); break;
      }
       gtk_label_set (GTK_LABEL (labelArg), buffer0);
       return;
 }

}
/*****************************
    Handle Unary Operations
 *****************************/
void HandleUnaryOperation (GtkWidget *widget, gpointer data)
{  
   char str[CMD_SIZE];
   strcpy (str, (char *) data);
   strcpy (lastStr, "u_op");
/*Here, `lastStr' is set to be "u_op".
 *Because labels should be cleared after unary operation.*/
   mv_cursor_real();   
   if (strcmp (str, "!") == 0)
   {
      if ((float) cz.i == 0.0 && -0.01 < cz.r)
      if (is_int(cz.r))
      if (cz.r < 2.0e9)
      {
         PrepareChange();
         cz.r = cz.abs =(double) factorial((int) yaround(cz.r));
         write_operation_stdout (UNARY_OP, str);/*defined in WriteOutput.c*/
         RenewLabels();
         return;
      }
      print_error();
      return;
   }
   if (strcmp (str, "!!") == 0)
   {
      if ((float) cz.i == 0.0 && -1.01 < cz.r)
      if (cz.r == -1.0 || is_int(cz.r))
      if (cz.r < 2.0e9)
      {
         PrepareChange();
         cz.r = (double) d_factorial((int) yaround(cz.r));
         absarg(&cz);
         write_operation_stdout (UNARY_OP, str);/*defined in WriteOutput.c*/
         RenewLabels();
         return;
      }
      print_error();
      return;
   }

   PrepareChange();

   if (strcmp (str, "asinh") == 0)
   {
      cz = zcasinh (cz);
   }
   else if (strcmp (str, "acosh") == 0)
   {
      cz = zcacosh (cz);
   }
   else if (strcmp (str, "atanh") == 0)
   {
      cz = zcatanh (cz);
   }
   else if (strcmp (str, "asin") == 0)
   {
      cz = zcasin (cz);
   }
   else if (strcmp (str, "acos") == 0)
   {
      cz = zcacos (cz);
   }
   else if (strcmp (str, "atan") == 0)
   {
      cz = zcatan (cz);
   }
   else if (strcmp (str, "sinh") == 0)
   {
      cz = zcsinh (cz);
   }
   else if (strcmp (str, "cosh") == 0)
   {
      cz = zccosh (cz);
   }
   else if (strcmp (str, "tanh") == 0)
   {
      cz = zctanh (cz);
   }
   else if (strcmp (str, "sin") == 0)
   {
      cz = zcsin (cz);
   }
   else if (strcmp (str, "cos") == 0)
   {
      cz = zccos (cz);
   }
   else if (strcmp (str, "tan") == 0)
   {
      cz = zctan (cz);
   }
   else if (strcmp (str, "log2") == 0)
   {
      zcomplex w = {2.0, 0.0, 2.0, 0.0};
      cz = zcdiv(zclog(cz), zclog(w));
   }
   else if (strcmp (str, "log10") == 0)
   {
      zcomplex w = {10.0, 0.0, 10.0, 0.0};
      cz = zcdiv(zclog(cz), zclog(w));
   }
   else if (strcmp (str, "ln") == 0)
   {
      cz = zclog(cz);
   }
   else if (strcmp (str, "sqrt") == 0)
   {
      zcomplex w = {0.50, 0.0, 0.50, 0.0};
      cz = zcpow(cz, w);
   }
   else if (strcmp (str, "cbrt") == 0)
   { 
      zcomplex w = {OneThird, 0.0, OneThird, 0.0};
      cz = zcpow(cz, w);
   }
   else if (strcmp (str, "exp") == 0)
   {
      cz = zcexp(cz);
   }
   else if (strcmp (str, "conj") == 0)
   {
      cz = zconj(cz);
   }
   else if (strcmp (str, "z^2") == 0)
   {
      zcomplex w = {2.0, 0.0, 2.0, 0.0};
      cz = zcpow(cz, w);
   }
   else if (strcmp (str, "z^3") == 0)
   {
      zcomplex w = {3.0, 0.0, 3.0, 0.0};
      cz = zcpow(cz, w);
   }
   else if (strcmp (str, "1/z") == 0)
   {
      zcomplex w = {1.0, 0.0, 1.0, 0.0};
      cz = zcdiv(w, cz);
   }
   else if (strcmp (str, "gamma") == 0)
   {
      cz = zcgamma(cz);
   }
   else if (strcmp (str, "re z") == 0)
   {
      cz.i = 0.0;
      absarg (&cz);
   }
   else if (strcmp (str, "im z") == 0)
   {
      cz.r = cz.i;
      cz.i = 0.0;
      absarg (&cz);
   }
   else if (strcmp (str, "|z|") == 0)
   {
      cz.r = cz.abs;
      cz.i = 0.0;
      cz.arg = 0.0;
   }
   else if (strcmp (str, "arg z") == 0)
   {
      cz.r = cz.arg;
      cz.i = 0.0;
      absarg (&cz);
   }

   write_operation_stdout (UNARY_OP, str);/*defined in WriteOutput.c*/
   RenewLabels();
   return;
}
/******************************
    Handle Binary Operations
 ******************************/
/*Executing previous command. Return 0 when failed.*/
static int exec_prevCmd (zcomplex *bufz);

void HandleBinaryOperation (GtkWidget *widget, gpointer data)
{
   int i, pr, must_del_op = 0;
   char str[CMD_SIZE];
   typCalculatorButton *clbt;

   /*The pointer onto clicked button*/
   clbt = (typCalculatorButton *)data;
   pr = clbt->pr;
   strcpy (str, clbt->szLabel);
   mv_cursor_real();

   PrepareChange();

   if (strcmp (str, "(") == 0)
   {
      if (p_count + 1 == N_P_B)
      {
         l_print_str("STACK ERROR");
         return;
      }
      handle_left_parenthesis (bufinputs);
   }
   /*I also wrote around parentheses near line 450.
    *Please see also "parenthesis.c" for detail.*/
   else if (!p_count && strcmp (str, ")") == 0)
   {
      print_error();
      return;
   }
   else if (p_count && strcmp (str, "=") == 0)
   {
      print_error();
      return;
   }
/*Execute previously input commands in stack one by one from highest
 *priority "1" to currently input command's priority "pr".*/
   for (i=0; i < pr; i++)
   {
      if (is_Command (bufinputs[i].cmd))
      {
         strcpy (prevCmd, bufinputs[i].cmd);
         /*Execute previous command.*/
         /*Get out when failed.*/
         if (exec_prevCmd (&(bufinputs[i].z)) == 0) return;
         /*Erase command that has been already executed.*/
         bufinputs[i].cmd[0] = '\0';
      }
   }

   /*And now we set current state to be previous state for next call.*/
   if (strcmp (str, ")") == 0)
   {
      write_operation_stdout (BINARY_OP, str);/*defined in WriteOutput.c*/
      handle_right_parenthesis (bufinputs);
      /*Here we restore the operator before '('.*/
      for (i = 0; i < N_PR; i++)
      {
         if (is_Command (bufinputs[i].cmd))
         {
            strcpy (str, bufinputs[i].cmd);
            break;
         }
      }
      /*We must clear the operator if it is meaningless.*/
      if (p_count == 0 && (*str == ')' || *str == '='))
         must_del_op = 1;
   }
   else
   {
      if (pr != 0)
      {
         if (strcmp (str, "nHr") == 0 ||
             strcmp (str, "nPr") == 0 ||
             strcmp (str, "nCr") == 0)
         {
            if ((float)cz.i != 0.0 || !is_int (cz.r))
            {
              print_error();
              return;
            }
            else if (cz.r < 0 || cz.r > 2.0e9)
            {
              print_error();
              return;
            }
         }
         /*Update the state of currently input command's level.*/
         bufinputs[pr-1].z = cz;
         strcpy (bufinputs[pr-1].cmd, str);
      }
      write_operation_stdout (BINARY_OP, str);/*defined in WriteOutput.c*/
   }
   strcpy (lastStr, str);
   strcpy (prevCmd, str);
   /*Update labels.*/
   RenewLabels();
   /*Update label for operator.*/
   RenewOp();

   if (must_del_op)
   {
      *prevCmd = '\0';
      gtk_label_set (GTK_LABEL (labelOp), NULL);
   }
   if (*str == '=')
   {
      write_operation_stdout (BINARY_OP, NULL);/*defined in WriteOutput.c*/
   }
   /*
   if (strcmp (str, "Hn*(z)") == 0)
   {
      gtk_label_set (GTK_LABEL (labelRe), "n = ? ");
      gtk_label_set (GTK_LABEL (labelIm), NULL);
      gtk_label_set (GTK_LABEL (labelAbs), NULL);
      gtk_label_set (GTK_LABEL (labelArg), NULL);
   }
   */
}
/*Executing previous command. Return 0 when failed.*/
static int exec_prevCmd (zcomplex *bufz)
{
   if (strcmp (prevCmd ,"mod") == 0)
   {
      *bufz = cz = zcfmod(*bufz, cz);
   }
   else if (strcmp (prevCmd, "/") == 0)
   {
      *bufz = cz = zcdiv(*bufz, cz);
   }
   else if (strcmp (prevCmd, "nHr") == 0)
   {
      if ((float)cz.i != 0.0 || !is_int (cz.r))
      {
         print_error();
         return 0;
      }
      else if (cz.r < 0 || cz.r > 2.0e9 || bufz->r < 1)
      {
         print_error();
         return 0;
      }
      
      cz.r = cz.abs = nHr ((int)yaround(bufz->r), (int)yaround(cz.r));
      *bufz = cz;
   }
   else if (strcmp (prevCmd, "nPr") == 0)
   {
      if ((float)cz.i != 0.0 || !is_int (cz.r))
      {
         print_error();
         return 0;
      }
      else if (cz.r < 0 || cz.r > 2.0e9 || bufz->r < cz.r)
      {
         print_error();
         return 0;
      }
      cz.r = cz.abs = nPr ((int)yaround(bufz->r), (int)yaround(cz.r));
      *bufz = cz;
   }
   else if (strcmp (prevCmd, "nCr") == 0)
   {
      if ((float)cz.i != 0.0 || !is_int (cz.r))
      {
         print_error();
         return 0;
      }
      else if (cz.r < 0 || cz.r > 2.0e9 || bufz->r < cz.r)
      {
         print_error();
         return 0;
      }
      
      cz.r = cz.abs = nCr ((int)yaround(bufz->r), (int)yaround(cz.r));
      *bufz = cz;
   }
   /*
   else if (strcmp (prevCmd, "Hn*(z)") == 0)
   {
      int n;
      if ((float)cz.r != (float)yaround(cz.r))
      {
        print_error();
        return 0;
      }
      else if ((n = (int)yaround(cz.r)) > max_n_of_hermite)
      {
        print_error();
        return 0;
      }
      *bufz = cz = Hn_s(n, *bufz);
   }
   */
   else if (strcmp (prevCmd, "z^w") == 0)
   {
      *bufz = cz = zcpow(*bufz, cz);
   }
   else if (strcmp (prevCmd, "*") == 0)
   {
      *bufz = cz = zcmul(*bufz, cz);
   }
   else if (strcmp (prevCmd, "+") == 0)
   {
      *bufz = cz = zcadd(*bufz, cz);
   }
   else if (strcmp (prevCmd, "-") == 0)
   {
      *bufz = cz = zcsub(*bufz, cz);
   }
   else if (strcmp (prevCmd, "=") == 0 || *prevCmd == '\0')
   {
      *bufz = cz;
      /*p_count = 0;*/
   }
   else/*if `prevCmd' is a parenthesis or someone that is not binary op.*/
   {
      *bufz = cz;
   }
   /*Return 1 when succeeded in executing the command.*/
   return 1;
}
/*****************************
    Handle Other Operations
 *****************************/
void HandleOthers (GtkWidget *widget, gpointer data)
{
   char str[CMD_SIZE];

   strcpy (str, (char *) data);/*get the label of clicked button here*/

    if (strcmp (str, "B.S.") == 0)/*BackSpace ---be careful about 'E'.*/
    {
        BSFunc();/*defined in "RenewLabels.c"*/
        return;
    }
    if (strcmp (str, "Cut") == 0)
    {
       char buf0[F_BUF_SIZE];
       /*The selection will be "__ + __i".*/
       create_str_for_copy (buf0, ' ');
       PrepareChange();
       cz.r = cz.i = cz.abs = cz.arg = 0.0;
       own_pr_sel (event_boxRe, buf0);
       lastStr[0] = '\0';
       /*buffer0 will be changed after "RenewLabels()".*/
       RenewLabels();
       mv_cursor_real();
       return;
    }
    if (strcmp (str, "Copy") == 0)
    {
       char buf0[F_BUF_SIZE] = "";
       /*The selection will be "__ + __i".*/
       create_str_for_copy (buf0, ' ');/*defined in WriteOutput.c*/
       own_pr_sel (event_boxRe, buf0);
       return;
    }
    if (strcmp (str, "Paste") == 0)
    {  
        clear_last_command ();
        request_pr_sel (event_boxRe);
        return;
    }
    if (strcmp (str, "CE/C") == 0)
    {
        PrepareChange();
        cz.r = cz.i = cz.abs = cz.arg = 0.0;
        p_count = 0;
        RenewLabels();
        lastStr[0] = '\0'; prevCmd[0] = '\0';
        clear_inputs (bufinputs);
        gtk_label_set (GTK_LABEL (labelOp), NULL);
        mv_cursor_real();
        write_operation_stdout (OTHERS, str);
        return;
    }
    if (strcmp (str, "Undo") == 0)
    {
        int i, is_hidden_m;
        if (n_memInputs == 0) return;
        /*UNDO is to pop buf_state.*/
        cz = buf_state[0].cz;
        mz = buf_state[0].mz;
        is_hidden_m = buf_state[0].is_hidden_m;
        strcpy (prevCmd, buf_state[0].prevCmd);
        copy_inputs (bufinputs, buf_state[0].inputs);
        if (p_count < buf_state[0].p_count)
           undo_parentheses (')');
        else if (p_count > buf_state[0].p_count)
           undo_parentheses ('(');
        /*reverse rotation*/
        for (i = 0; i < N_MI - 1; i++)
        {
           buf_state[i].cz = buf_state[i+1].cz;
           buf_state[i].mz = buf_state[i+1].mz;
           buf_state[i].is_hidden_m = buf_state[i+1].is_hidden_m;
           buf_state[i].p_count = buf_state[i+1].p_count;
           copy_inputs(buf_state[i].inputs, buf_state[i+1].inputs);
           strcpy (buf_state[i].prevCmd, buf_state[i+1].prevCmd);
        }
        RenewLabels();
        RenewMem();
        RenewOp();
        if (is_hidden_m && mz.r == 0.0 && mz.i == 0.0)
           gtk_label_set (GTK_LABEL (labelMem), NULL);
        strcpy (lastStr, "u_op");

        n_memInputs--;
        mv_cursor_real();
        write_operation_stdout (OTHERS, str);
        return;
    }
    /*All Copy*/
    if (strcmp (str, "ACP") == 0)
    {
       char buf0[F_BUF_SIZE] = "";
       /*The selection will be "__ + __i = __ ej __".*/
       create_str_for_copy (buf0, 'a');
       own_pr_sel (event_boxRe, buf0);
       return;
    }
    /*All Clear*/
    if (strcmp (str, "ACL") == 0)
    {
        PrepareChange();
        cz.r=0.0; cz.i=0.0; cz.abs=0.0; cz.arg=0.0;      
        mz.r=0.0; mz.i=0.0; mz.abs=0.0; mz.arg=0.0;
        p_count = 0;
        RenewLabels();
        lastStr[0] = '\0'; prevCmd[0] = '\0';
        clear_inputs (bufinputs);
        gtk_label_set (GTK_LABEL (labelMem), NULL);
        gtk_label_set (GTK_LABEL (labelOp), NULL);
        mv_cursor_real();
        write_operation_stdout (OTHERS, str);
        return;
    }
    /*Rotate unit for expressing argument.*/
    if (strcmp (str, "DRPI") == 0)
    {
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
        switch(arg_unit)
        {
          case RADIAN:
            arg_unit = PIRADIAN;
            RenewArg();
            return;
          case PIRADIAN:
            arg_unit = DEGREE;
            RenewArg();
            return;
          default:
            arg_unit = RADIAN;
            RenewArg();
            return;
        }
#else /*GTK+-1.2*/
        switch(arg_unit)
        {
          case RADIAN:
            arg_unit = PIRADIAN;
            RenewArg();
            return;
          case PIRADIAN:
            arg_unit = DEGREE;
            RenewArg();
            return;
          default:
            arg_unit = RADIAN;
            RenewArg();
            return;
        }
 #endif /*GTK+-X.X*/
    }
/*Memory operations*/
/*Treat an operation on the memory as a unary operation.*/
   PrepareChange();
   strcpy (lastStr, "u_op");
   /*mv_cursor_real();*/
    if (strcmp (str, "STO") == 0)
    {  
        mz = cz;
        RenewMem();
        return;
    }
    if (strcmp (str, "RCL") == 0)
    {  
        cz = mz;
        RenewLabels();
        return;
    }
    if (strcmp (str, "M+") == 0)
    {  
        mz = zcadd(mz, cz);
        RenewMem();
        return;
    }
    if (strcmp (str, "M-") == 0)
    {  
        mz = zcsub(mz, cz);
        RenewMem();
        return;
    }
    if (strcmp (str, "M*") == 0)
    {  
        mz = zcmul(mz, cz);
        RenewMem();
        return;
    }
    if (strcmp (str, "M/") == 0)
    {  
        mz = zcdiv(mz, cz);
        RenewMem();
        return;
    }
    if (strcmp (str, "M^") == 0)
    {  
        mz = zcpow(mz, cz);
        RenewMem();
        return;
    }
    if (strcmp (str, "conjM") == 0)
    {  
        mz = zconj(mz);
        RenewMem();
        return;
    }

}
