/*a simple calculator for complex number system
 *Akira Saitoh (1999,2002,2006)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 * [3] Hiroyuki Tanaka: GTK+ nyuumon (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include <stdlib.h>
#include "cgtkcalc.h"
#include "CmdlineOptions.h"
#include "WriteOutput.h"
#include "RenewLabels.h"

/*Make main window shrunk one.*/
void shrink_mwindow (GtkWidget *table, GtkWidget *hbox, GtkWidget *mwindow)
{
   int i;

   gtk_widget_hide (hbox);
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   {
      extern GtkWidget *HelpButton;/*defined in CreateToolbar.c*/
      gtk_widget_realize (HelpButton);
   }
#endif /*GTK+-2.0*/
   gtk_table_set_homogeneous (GTK_TABLE (table), FALSE);
   for (i = 0; i < 8; i++)
   {
      gtk_widget_hide (buttonList[i].widget);
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
      gtk_widget_realize (buttonList[i].widget);
#endif /*GTK+-2.0*/
   }
   for (i = 12; i < nButtons; i++)
   {
      gtk_widget_hide (buttonList[i].widget);
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
      gtk_widget_realize (buttonList[i].widget);
#endif /*GTK+-2.0*/
   }
   gtk_window_set_policy (GTK_WINDOW (mwindow), FALSE, FALSE, FALSE);
   gtk_window_set_title (GTK_WINDOW (mwindow), "cgtkcalc --shrunk");
}

/*Use a cursor which looks like a hand.*/
void use_hand_cursor (GtkWidget *table)
{
   gdk_window_set_cursor (table->window, gdk_cursor_new (GDK_HAND1));
}

/*Use PIrad for expressing argument.*/
void start_with_PIrad (void)
{
   arg_unit = PIRADIAN;
}

/*Use deg for expressing argument.*/
void start_with_deg (void)
{
   arg_unit = DEGREE;
}

/*Hide absolute value and argument.*/
void hide_absarg (GtkWidget *table)
{
   int i;
   GtkWidget *cover;/*a cosmetic cover*/
   GtkWidget *label;

   /*Hide the "DRPI" button.*/
   cover = gtk_fixed_new ();
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   gtk_fixed_set_has_window (GTK_FIXED (cover), TRUE);
#endif /*GTK+-2.0*/
   gtk_table_attach (GTK_TABLE (table), cover,
                     5, 6, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
   gtk_widget_show (cover);
   /*Hide absolute value and argument.*/
   for (i = 0; i < 8; i++)
   {
      cover = gtk_fixed_new ();
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
      gtk_fixed_set_has_window (GTK_FIXED (cover), TRUE);
#endif /*GTK+-2.0*/
      gtk_table_attach (GTK_TABLE (table), cover,
                        i, i + 1, 2, 3, GTK_FILL, GTK_FILL, 0, 2);
      label = gtk_label_new ("");
      gtk_fixed_put (GTK_FIXED (cover), label, 50, 0);
      gtk_widget_show (label);
      gtk_widget_show (cover);
   }
   gtk_table_set_homogeneous (GTK_TABLE (table), FALSE);
   gtk_table_set_row_spacing (GTK_TABLE (table), 2, 0);
   gtk_widget_hide (labelAbs);
   gtk_widget_hide (labelArg);
   gtk_widget_hide (event_boxAbs);
   gtk_widget_hide (event_boxArg);
   gtk_widget_hide (buttonList[NB_DRPI].widget);
   gtk_widget_hide (buttonList[NB_ABS].widget);
   gtk_widget_hide (buttonList[NB_ARG].widget);
   /*Change the pointers to disable abs. and arg. from entering.*/
   buttonList[NB_ABS].widget = buttonList[NB_REAL].widget;
   buttonList[NB_ARG].widget = buttonList[NB_IMAG].widget;
}

/*Use the specified rc file, e.g, ~/mygtkrc.*/
void use_specified_rc (char *str, GtkWidget *window)
{
   FILE *rcfile;
   if ((rcfile = fopen (str, "r")))
   {
      fclose (rcfile);
      gtk_rc_parse (str);
      gtk_widget_reset_rc_styles (window);
   }
}

/*Control command line options here.*/
void handle_command_line_options (int _argc, char *_argv[],
                                  GtkWidget *mwindow,
                                  GtkWidget *table, GtkWidget *hbox,
                                  int *stv)
{
   int i;
   int already_shrunk = 0, already_hidden = 0;

   if (1 < _argc && _argc < 20)
      for (i = 1; i < _argc; i++)
      {
      /*Shrunk mode*/
         if ((strcmp (_argv[i], "--shrunk") == 0  ||
              strcmp (_argv[i], "--simple") == 0) &&
             !already_shrunk)
         {
            shrink_mwindow (table, hbox, mwindow);
            already_shrunk = 1;
         }
      /*Hide abs and arg*/
         else if (strcmp (_argv[i], "--hide") == 0 &&
                  !already_hidden)
         {
            hide_absarg (table);
            already_hidden = 1;
         }
      /*PIrad mode*/
         else if (strcmp (_argv[i], "--PIrad") == 0)
         {
            start_with_PIrad ();
         }
      /*deg mode*/
         else if (strcmp (_argv[i], "--deg") == 0)
         {
            start_with_deg ();
         }
      /*Let the argument be in [0, 2PI)*/
         else if (strcmp (_argv[i], "--2pi") == 0)
         {
            bt0_2PI = 1;
         }
      /*Hand cursor mode*/
         else if (strcmp (_argv[i], "--Hand") == 0)
         {
            use_hand_cursor (table);
         }
      /*Output to stdout*/
         else if (strcmp (_argv[i], "--record") == 0)
         {
            var_write_ops = 1;
         }
      /*Select a font family.*/
         else if (strcmp (_argv[i], "--font") == 0 ||
                  strcmp (_argv[i], "-f") == 0 )
         {
            if (_argc != i + 1)
            {
               if (strcmp (_argv[i+1], "fixed") == 0 )
                  *stv = F_FIXED;
               else if (strcmp (_argv[i+1], "century") == 0 )
                  *stv = F_CENTURY;
               else if (strcmp (_argv[i+1], "helvetica") == 0 )
                  *stv = F_HELVETICA;
               else if (strcmp (_argv[i+1], "times") == 0 )
                  *stv = F_TIMES;

               else if (strcmp (_argv[i+1], "big") == 0 && *stv < 4)
                  *stv += 4;
            }
            if (_argc != i + 2 && *stv < 4)
            {
               if (strcmp (_argv[i+2], "big") == 0)
                  *stv += 4;
            }
         }
      /*Font color for real part and imaginary part.*/
         else if (strcmp (_argv[i], "-fc-real") == 0)
         {
            if (_argc != i + 1)
            {
               gdk_color_parse (_argv[i+1], &fc_real);
            }
         }
      /*Font color for absolute value and argument.*/
         else if (strcmp (_argv[i], "-fc-abs") == 0)
         {
            if (_argc != i + 1)
            {
               gdk_color_parse (_argv[i+1], &fc_abs);
            }
         }
      /*Font color for operation label and memory label.*/
         else if (strcmp (_argv[i], "-fc-op") == 0)
         {
            if (_argc != i + 1)
            {
               gdk_color_parse (_argv[i+1], &fc_op);
            }
         }
      /*The precision specifies the number of siginificant digits on labels.*/
         else if (strcmp (_argv[i], "-prec") == 0 ||
                  strcmp (_argv[i], "--prec") == 0)
         {
            if (_argc != i + 1)
            {
               int p;
               p = atoi (_argv[i+1]);
               if (0 < p && p < 12)
                  l_precision = p;/*l_precision is defined in "RenewLabels.c".*/
            }
         }
      /*Use the specified rc file.*/
         else if (strcmp (_argv[i], "--rc") == 0 ||
                  strcmp (_argv[i], "-rc") == 0)
         {
            if (_argc != i + 1)
            {
               use_specified_rc (_argv[i+1], mwindow);
            }
         }
      /*Cascade of options except font options.*/
         else if (*(_argv[i]) == '-')
         {
            int j, len;
            len = strlen (_argv[i]);
            for (j = 1; j < len && j < 10; j ++)
            {
               switch (*(_argv[i] + j))
               {
                  /*Shrunk mode*/
                  case 's':
                     if (!already_shrunk)
                     {
                        shrink_mwindow (table, hbox, mwindow);
                        already_shrunk = 1;
                     }
                     break;
                  /*Hide abs and arg*/
                  case 'i':
                     if (!already_hidden)
                     {
                        hide_absarg (table);
                        already_hidden = 1;
                     }
                     break;
                  /*PIrad mode*/
                  case 'p':
                     start_with_PIrad ();
                     break;
                  /*deg mode*/
                  case 'd':
                     start_with_deg ();
                     break;
                  /*Hand cursor mode*/
                  case 'H':
                     use_hand_cursor (table);
                     break;
                  /*Let the argument be in [0, 2PI)*/
                  case 'D':
                     bt0_2PI = 1;
                     break;
                  /*Output to stdout*/
                  case 'w':
                     var_write_ops = 1;
                     break;
               }
            }
         }
      }
}
