/*-----------cgtkcalc----------------
 *a simple calculator for complex number system
 *Akira Saitoh (1999,2002)
 *---------------------------------
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include <string.h>
#include <stdlib.h>
#include "cgtkcalc.h"
#include "RenewLabels.h"
#include "Copy.h"
#include "zc_str.h"

#ifdef FOR_MINGW32
#include <windows.h>
#endif /*FOR_MINGW32*/

/*A string to be pasted*/ 
char* CopiedStr = NULL;
/*NOTE: This will be freed in a function written in cgtkcalc.c.*/

/*With this function, a widget owns a selection.*/
void own_pr_sel (GtkWidget *widget, char *str)
{
/* Make `CopiedStr' independent of `str'.
 * Its independence is necessary to work correctly.
 * And trim its leading white space here.
 */
   CopiedStr = (char *) realloc (CopiedStr, (strlen(str) + 1) * sizeof(char));
   if (CopiedStr == NULL) return;
   strcpy (CopiedStr, str);
   g_strchug (CopiedStr);
   gtk_selection_owner_set (widget, GDK_SELECTION_PRIMARY,
                            GDK_CURRENT_TIME);

/*The block below is only for the win32 environment.*/
/*Copy the string onto the Clipboard.*/
#ifdef FOR_MINGW32
   {
      HGLOBAL htxt;
      HANDLE htxt2;
      char* ptxt = NULL;
      char* ptxt2 = NULL;
      int i;
      /* It sometimes happens that the clipboard forgets the string
       * just pasted. For a workaround, we will take a second trial
       * if it happens.*/
      for (i = 0; i < 2; i++)
      {
        htxt = GlobalAlloc (GMEM_DDESHARE | GMEM_MOVEABLE, 
                            (strlen(CopiedStr) + 1) * sizeof(char));
        if (htxt)
        {
           ptxt = (char*)GlobalLock (htxt);
           if (ptxt)
           {
              strcpy (ptxt, CopiedStr);
              GlobalUnlock (htxt);
              if (OpenClipboard (NULL))
	      {
                 EmptyClipboard ();
                 SetClipboardData (CF_TEXT, htxt);
                 CloseClipboard ();
              }
           }
           else
	   {
              GlobalUnlock (htxt);
           }
        }
        GlobalFree (htxt);
        if (i == 1) break;
        /* Check if the first trial has worked fine.*/
        if (OpenClipboard (NULL))
        {
          htxt2 = GetClipboardData (CF_TEXT);
          if (htxt2 != NULL)
          {
            ptxt2 = (char *)GlobalLock (htxt2);
            GlobalUnlock (htxt2);
            if (ptxt2 != NULL && ptxt != NULL)
            {
               if (ptxt2[0] == ptxt[0]) i++;
            }
          }
          CloseClipboard ();
        }
      }
   }
#endif /*FOR_MINGW32*/
}

/*With this function, a widget requests the data from the owner,
  and waits for `selection_received' event.*/
void request_pr_sel (GtkWidget *widget)
{
/*The block below is only for the win32 environment.*/
/*Paste the string obtained from the Clipboard.*/
#ifdef FOR_MINGW32
   {
      HANDLE htxt;
      char* ptxt;
      if (OpenClipboard (NULL))
      {
         htxt = GetClipboardData (CF_TEXT);
         if (htxt != NULL)
         {
	    ptxt = (char*)GlobalLock (htxt);
            GlobalUnlock (htxt);
            complete_paste_func (ptxt);
            CloseClipboard ();
            return;
         }
         CloseClipboard ();
      }
   }
#endif /*FOR_MINGW32*/

    gtk_selection_convert (widget, 
			               GDK_SELECTION_PRIMARY,
			               GDK_SELECTION_TYPE_STRING,
			               GDK_CURRENT_TIME);
}

/*When the widget obtained a selection, this function is called
  to set a datum.*/
void selection_obtained (GtkWidget *widget, GtkSelectionData *selection_data,
                         guint info, guint time_stamp, gpointer data )
{
   gtk_selection_data_set (selection_data, GDK_SELECTION_TYPE_STRING,
                           8, (guchar *)CopiedStr, strlen(CopiedStr));
   /*Only for debug.*/
   /*puts (CopiedStr);*/
}

/*Deal with received datum.*/
void selection_data_received (GtkWidget *widget, GtkSelectionData *selection_data, gpointer data)
{
    if (selection_data->length < 0)
        return;
    if (selection_data->type != GDK_SELECTION_TYPE_STRING)
        return;
    complete_paste_func ((char *)selection_data->data);
}

/*This function is called after `PASTE' button gets 
  `selection_received' event successfully.*/
void complete_paste_func (char *buffer0)
{
   if (is_zc_str (buffer0))
   {
      cz = a2zc (buffer0);/*"alphabet to zcomplex"*/
      RenewLabels();
      return;
   }
   switch (inputpos)
   {
      case REAL:
         cz.r = atof (buffer0); absarg (&cz);
         RenewLabels();
         return;
      case IMAG:
         cz.i = atof (buffer0); absarg (&cz);
         RenewLabels();
         return;
      case ABSO:
         cz.abs = fabs (atof (buffer0)); realimag (&cz);
         RenewLabels();
         return;
      case ARGU:
        switch (arg_unit){
         case RADIAN: cz.arg = atof (buffer0); break;
        /*translate PIrad into rad*/
         case PIRADIAN: cz.arg = tPI * atof (buffer0); break;
        /*translate deg into rad*/
         case DEGREE: cz.arg = (tPI / 180.0 * atof (buffer0)); break;
        }
         realimag (&cz);
         RenewLabels();
         return;
   }
}

/*Set up event_boxRe to handle selections.*/
void setup_selection (void)
{
   gtk_selection_add_target (event_boxRe,
                             GDK_SELECTION_PRIMARY,
                             GDK_SELECTION_TYPE_STRING,
                             1);
   gtk_signal_connect (GTK_OBJECT (event_boxRe), "selection_get",
                       GTK_SIGNAL_FUNC (selection_obtained), NULL);
   gtk_signal_connect (GTK_OBJECT (event_boxRe), "selection_received",
                       GTK_SIGNAL_FUNC (selection_data_received), NULL);
}
