/*
 *a simple calculator for complex number system
 *Akira Saitoh (1999,2002,2015)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "Clicked.h"
#include "Copy.h"
#include "CreateButtons.h"
static GtkWidget *CreateButton (GtkWidget *table, int nIndex);
static GtkWidget *CreateToggleButton (GtkWidget *table, int nIndex);
static GtkWidget *CreateCover (GtkWidget *table, int nIndex);

void CreateCalculatorButtons (GtkWidget *table)
{
   int nIndex;
  
   for (nIndex = 0; nIndex < nButtons; nIndex++)
   {
      if (buttonList[nIndex].sort == TOGGLE_B)
      {
         buttonList[nIndex].widget = CreateToggleButton (table, nIndex);
      }
      else if (buttonList[nIndex].sort == COVERS)
      {
         buttonList[nIndex].widget = CreateCover (table, nIndex);
      }
      else
      {
         buttonList[nIndex].widget = CreateButton (table, nIndex);
      }
   }

   return;
}

static GtkWidget *CreateButton (GtkWidget *table, int nIndex)
{
   GtkWidget *button;
   int row, col, width;
   char *szLabel;

   row = buttonList[nIndex].row;
   col = buttonList[nIndex].col;
   width = buttonList[nIndex].width;
   szLabel = buttonList[nIndex].szLabel;

   button = gtk_button_new_with_label (szLabel);
  switch (buttonList[nIndex].sort)
  {
    case DIGITS:/*digit*/
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (HandleDigit), szLabel);
       break;
    case UNARY_OP:/*unary operator*/
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (HandleUnaryOperation), szLabel);
       break;
    case BINARY_OP:/*binary operator*/
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (HandleBinaryOperation),
                           (gpointer) &buttonList[nIndex]);
       break;
    case OTHERS:/*others*/
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (HandleOthers), szLabel);
       break;
    default:
       break;
  }
   gtk_table_attach (GTK_TABLE (table), button,
                     col, col + width,
                     row, row + 1,
                     GTK_FILL | GTK_EXPAND,
                     GTK_FILL | GTK_EXPAND,
                     1, 1);
   gtk_widget_show (button); 
   return (button);
}

static GtkWidget *CreateToggleButton (GtkWidget *table, int nIndex)
{
   GtkWidget *button;
   int row, col, width;
   char *szLabel;

   row = buttonList[nIndex].row;
   col = buttonList[nIndex].col;
   width = buttonList[nIndex].width;
   szLabel = buttonList[nIndex].szLabel;

   button = gtk_toggle_button_new_with_label (szLabel);
   gtk_signal_connect (GTK_OBJECT (button), "pressed",
                       GTK_SIGNAL_FUNC (button_pressed), szLabel);
   gtk_signal_connect (GTK_OBJECT (button), "released",
                       GTK_SIGNAL_FUNC (button_released), szLabel);
   gtk_table_attach (GTK_TABLE (table), button,
                     col, col + width,
                     row, row + 1,
                     GTK_FILL | GTK_EXPAND,
                     GTK_FILL | GTK_EXPAND,
                     5, 5);
   gtk_widget_show (button);
   return (button);
}

static GtkWidget *CreateCover (GtkWidget *table, int nIndex)
{
   GtkWidget *cover;
   GtkWidget *line;
   int row, col, width;
   /* char *szLabel; */
   int i;

   row = buttonList[nIndex].row;
   col = buttonList[nIndex].col;
   width = buttonList[nIndex].width;
   /* szLabel = buttonList[nIndex].szLabel; */

   cover = gtk_vbox_new (TRUE, 1);
   gtk_table_attach (GTK_TABLE (table), cover, col, col + width,
                     row, row + 1, GTK_FILL, GTK_FILL, 0, 1);
   for (i = 0; i < 12; i++)
   {
      line = gtk_fixed_new ();
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
      gtk_fixed_set_has_window (GTK_FIXED (line), TRUE);
#endif /*GTK+-2.0*/
      gtk_box_pack_end (GTK_BOX (cover), line, TRUE, TRUE, 0);
      gtk_widget_show (line);
   }
   gtk_widget_show (cover);
   return (cover);
}

