/*a simple calculator for complex number system
 *Akira Saitoh (1999-2017)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "DEF_VERSION.h"
#include "CreateHelpWindow.h"
#include "WindowIcon.h"

/*I will never use `gettext' for i18n!*/
/*DEF_SHARED_HELP.h will be regenerated by configure.*/
#include "DEF_SHARED_HELP.h"
/*Maximum number of bytes used for HELP.*/
#define N_MAX_STR 2222

typedef struct
{
   GtkWidget *dialog;
   GtkWidget *swindow;
   GtkWidget *label; 
   GtkWidget *button;
   char *str;
} HelpBox;

static HelpBox helpbox = {NULL,NULL,NULL,NULL,NULL};
static void close_help (GtkWidget *widget, gpointer data);

void CreateHelpWindow (GtkWidget *widget, gpointer data)
{
   FILE *file;
   int nch = 0;
   char *lstr = NULL;
   /* Avoid opening multiple windows.*/
   if (helpbox.dialog != NULL) return;
   /* Allocate memory to string for help function. */ 
   helpbox.str = (char *) malloc (N_MAX_STR * sizeof (char));
   if (helpbox.str == NULL) return;

   /* Create dialog window. */
   helpbox.dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (helpbox.dialog), _VERSION_HELP_STR);
   setup_window_icon (GTK_WINDOW (helpbox.dialog));/*defined in "WindowIcon.c"*/
   gtk_widget_set_usize (helpbox.dialog, 400, 255);
   gtk_signal_connect (GTK_OBJECT (helpbox.dialog), "destroy",
                       GTK_SIGNAL_FUNC (close_help), NULL);

   /* Create a scrolled_window on the dialog window. */
   helpbox.swindow = gtk_scrolled_window_new (NULL,NULL);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (helpbox.dialog)->vbox),
                       helpbox.swindow, TRUE, TRUE, 0);
   gtk_widget_show (helpbox.swindow);

#ifdef FOR_MINGW32
#include "HELP_STR_MINGW32.include"
   strncpy (helpbox.str, HELP_STR_MINGW32, N_MAX_STR - 1);
   *(helpbox.str + N_MAX_STR - 1) = '\0';
#else /*if FOR_MINGW32 is not defined*/

   /* Handle $LANG and $LANGUAGE */
   if ((lstr = getenv ("LANG")))
   {
      if (strcmp ((const char*) lstr, "ja_JP.eucJP") != 0 &&
          (strcmp ((const char*) lstr, "ja_JP.UTF-8") != 0 &&
           strcmp ((const char*) lstr, "ja_JP.utf8") != 0))
         if ((lstr = getenv ("LANGUAGE")))
            if (strcmp ((const char*) lstr, "ja_JP.eucJP") != 0 &&
                (strcmp ((const char*) lstr, "ja_JP.UTF-8") != 0 &&
                 strcmp ((const char*) lstr, "ja_JP.utf8") != 0))
               lstr = NULL;
   }
   /* Get a string from $prefix/share/cgtkcalc/HELP_STR_EUC_JP 
    * or from $prefix/share/cgtkcalc/HELP_STR_UTF_8_JP        */
   if (lstr)
   {
      if (*lstr == 'j')
      {
         if ((file = fopen(_HELP_FILE_JP, "rb")) != NULL)
         {
            nch = fread (helpbox.str, sizeof (char), N_MAX_STR - 1, file);
            fclose (file);
            *(helpbox.str + nch) = '\0';
         }
         else
         {
            strncpy (helpbox.str, _HELP_FILE_JP, N_MAX_STR - 12);
            strncat (helpbox.str, " NOT FOUND.", 12);
         }
      }
   }
   /* Get string from $prefix/share/cgtkcalc/HELP_STR */
   else
   { 
      if ((file = fopen(_HELP_FILE, "rb")) != NULL)
      {
         nch = fread (helpbox.str, sizeof (char), N_MAX_STR - 1, file);
         fclose (file);
         *(helpbox.str + nch) = '\0';
      }
      else
      {
         strncpy (helpbox.str, _HELP_FILE, N_MAX_STR - 12);
         strncat (helpbox.str, " NOT FOUND.", 12);
      }
   }
#endif /*if FOR_MINGW32 is not defined*/

   /* Create a label on the dialog window. */ 
   helpbox.label = gtk_label_new (helpbox.str);
   gtk_misc_set_alignment (GTK_MISC (helpbox.label), .5, .5);
   gtk_label_set_pattern (GTK_LABEL (helpbox.label), " ________");
   gtk_label_set_justify (GTK_LABEL (helpbox.label), GTK_JUSTIFY_LEFT);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(helpbox.swindow),
                                          helpbox.label);
   gtk_widget_show (helpbox.label);

   /* Create a button "OK" on the dialog window. */    
   helpbox.button = gtk_button_new_with_label ("  OK  ");
   GTK_WIDGET_SET_FLAGS (helpbox.button, GTK_CAN_DEFAULT);
   gtk_signal_connect (GTK_OBJECT (helpbox.button),"clicked",
                       GTK_SIGNAL_FUNC (close_help), NULL);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (helpbox.dialog)->action_area),
                       helpbox.button, FALSE, FALSE, 0);
   gtk_widget_show (helpbox.button);
   gtk_widget_grab_default (helpbox.button);

   gtk_widget_show (helpbox.dialog);
}

static void close_help (GtkWidget *widget, gpointer data)
{
   gtk_label_set_text (GTK_LABEL (helpbox.label), NULL);
   /* We must free the allocated memory. */
   free_helpbox_str ();

   gtk_widget_destroy (helpbox.dialog);
   helpbox.dialog = NULL;
}

void free_helpbox_str ()
{
   if (helpbox.str != NULL)
   {
      free (helpbox.str);
      helpbox.str = NULL;
   }
}
