/*a simple calculator for complex number system
 *Akira Saitoh (1999,2005)
 *
 *References
 * [1] Hiroyuki Tanaka: GTK+ nyuumon (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "cgtkcalc.h"
#include "CreateHbox.h"
#include "CreateHelpWindow.h"
#include "RenewLabels.h"

#define SEPARATOR -32

GtkWidget *HelpButton;
GtkWidget *FontMenu;
GtkWidget *ArgMenu;

static GtkWidget *font_menu();
static void font_change (GtkWidget *widget, gpointer f);
static GtkWidget *arg_menu();
static void arg_domain_init (GtkWidget *widget, gpointer f);

void CreateToolbar(GtkWidget *toolbar, GtkWidget *handle)
{
   GtkWidget *QuitButton;
   
   QuitButton = gtk_button_new_with_label ("Quit");
   gtk_widget_set_usize (QuitButton, 0, 30);
   gtk_signal_connect (GTK_OBJECT(QuitButton), "clicked",
                       GTK_SIGNAL_FUNC(QuitApp),NULL);
   HelpButton = gtk_button_new_with_label ("Help");
   gtk_widget_set_usize (HelpButton, 0, 30);
   gtk_signal_connect (GTK_OBJECT(HelpButton), "clicked",
                       GTK_SIGNAL_FUNC(CreateHelpWindow),NULL);

   FontMenu = font_menu();
   gtk_widget_set_usize (FontMenu, 46, 30);
   
   ArgMenu = arg_menu();
   gtk_widget_set_usize (ArgMenu, 46, 30);

#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   toolbar = gtk_toolbar_new ();
#else /*GTK+-1.2*/
   toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
#endif /*GTK+-X.X*/
   gtk_toolbar_append_widget (GTK_TOOLBAR(toolbar), QuitButton, "Good-bye.", NULL);
   gtk_toolbar_append_widget (GTK_TOOLBAR(toolbar), HelpButton, NULL, NULL);
   gtk_toolbar_append_widget (GTK_TOOLBAR(toolbar), FontMenu, "Font", NULL);
   gtk_toolbar_append_widget (GTK_TOOLBAR(toolbar), ArgMenu,
                              "Domain of the argument", NULL);
   
   gtk_container_add (GTK_CONTAINER(handle), GTK_WIDGET(toolbar));

   gtk_widget_show (QuitButton);
   gtk_widget_show (HelpButton);
   gtk_widget_show (FontMenu);
   gtk_widget_show (ArgMenu);
   gtk_widget_show (toolbar); 
}
/*a menu to select a font for labels*/
static GtkWidget *font_menu()
{  
   GtkWidget *optMenu;
   GtkWidget *fontMenu;
   GtkWidget *item; 

   optMenu = gtk_option_menu_new ();
   fontMenu = gtk_menu_new ();
   
     item = gtk_menu_item_new_with_label ("F      Font");
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new (); /*Create separator.*/
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) SEPARATOR);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Century");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) F_CENTURY);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Fixed");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) F_FIXED);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item); 

     item = gtk_menu_item_new_with_label ("Helvetica");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) F_HELVETICA);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item); 

     item = gtk_menu_item_new_with_label ("Times");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) F_TIMES);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Century Big");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) BIG_CENTURY);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Fixed Big");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) BIG_FIXED);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Helvetica Big");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) BIG_HELVETICA);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("Times Big");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (font_change), (gpointer) BIG_TIMES);
     gtk_menu_append (GTK_MENU (fontMenu), item);
     gtk_widget_show (item);

   gtk_option_menu_set_menu (GTK_OPTION_MENU (optMenu), fontMenu);
   
  return (optMenu);
}

static void font_change (GtkWidget *widget, gpointer data)
{
   GtkStyle *newstyle;
   int stv = (int)(gulong)data;

   if (stv != SEPARATOR)
   {
      newstyle = CreateStyle (stv);
      /*Labels for current values use the same style.*/
      gtk_widget_set_style (labelRe, newstyle);
      gtk_widget_set_style (labelIm, newstyle);
      gtk_style_unref (newstyle);

      newstyle = CreateStyle (stv ^ M_ABS);
      /*Labels for current values use the same style.*/
      gtk_widget_set_style (labelAbs, newstyle);
      gtk_widget_set_style (labelArg, newstyle);
      gtk_style_unref (newstyle);

      newstyle = CreateStyle (stv ^ M_OP);
      /*Labels for memorized value and operation use the same style.*/
      gtk_widget_set_style (labelMem, newstyle);
      gtk_widget_set_style (labelOp, newstyle);
      gtk_style_unref (newstyle);
   }
   gtk_option_menu_set_history (GTK_OPTION_MENU (FontMenu), 0);
}

/*a menu to select the domain of argument value*/
static GtkWidget *arg_menu()
{  
   GtkWidget *optMenu;
   GtkWidget *argMenu;
   GtkWidget *item; 

   optMenu = gtk_option_menu_new ();
   argMenu = gtk_menu_new ();
   
     item = gtk_menu_item_new_with_label ("D      Domain");
     gtk_menu_append (GTK_MENU (argMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new (); /*Create separator.*/
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (arg_domain_init), (gpointer) SEPARATOR);
     gtk_menu_append (GTK_MENU (argMenu), item);
     gtk_widget_show (item);
 
     item = gtk_menu_item_new_with_label ("(-PI, PI]");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (arg_domain_init), (gpointer) 0);
     gtk_menu_append (GTK_MENU (argMenu), item);
     gtk_widget_show (item);

     item = gtk_menu_item_new_with_label ("[ 0, 2PI)");
     gtk_signal_connect (GTK_OBJECT (item), "activate", 
                         GTK_SIGNAL_FUNC (arg_domain_init), (gpointer) 1);
     gtk_menu_append (GTK_MENU (argMenu), item);
     gtk_widget_show (item);

   gtk_option_menu_set_menu (GTK_OPTION_MENU (optMenu), argMenu);
   
   return (optMenu);
}
/*
 *The argument value is between -PI and PI by default.
 *If that between 0 and 2PI is needed, bt0_2PI should be 1.
 */
static void arg_domain_init (GtkWidget *widget, gpointer f)
{
   if ((gulong)f == 0)       bt0_2PI = 0;
   else if ((gulong)f == 1)  bt0_2PI = 1;
   gtk_option_menu_set_history (GTK_OPTION_MENU (ArgMenu), 0);
   absarg (&cz);
   RenewArg();
}
