/*a simple calculator for complex number system
 *Akira Saitoh (1999,2002)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "KeyPressed.h"
/*see also <gdk/gdkkeysyms.h>*/
#define GDK_BackSpace 0xFF08
#define GDK_Delete 0xFFFF
#define GDK_KP_Delete 0xFF9F
#define GDK_Return 0xFF0D
#define GDK_KP_Enter 0xFF8D

extern GtkWidget *FontMenu;/*defined in CreateToolbar.c*/
extern GtkWidget *HelpButton;/*defined in CreateToolbar.c*/
extern GtkWidget *ArgMenu;/*defined in CreateToolbar.c*/

/* Handle keyboard shortcuts.
   NB_* is defined in "ButtonList.h". */
void key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  int nIndex;
  guint kval = event->keyval;
 /*If input from key-pad, shift it to ordinary value.*/
  if (0xFFA9 < kval && kval < 0xFFBE)
        kval -= 0xFF80;

  if (event->state & GDK_CONTROL_MASK)/*If 'Ctrl' key is being pressed,*/
  {
    switch (kval)
    {   
     case 'x':case 'X':/*CUT*/
       gtk_widget_grab_focus (buttonList[NB_CUT].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_CUT].widget));
       return;
     case 'c':case 'C':/*COPY*/
       gtk_widget_grab_focus (buttonList[NB_COPY].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_COPY].widget));
       return;
     case 'v':case 'V':/*PASTE*/
       gtk_widget_grab_focus (buttonList[NB_PASTE].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_PASTE].widget));
       return;
     case 'u':case 'U':case 'z':case 'Z':/*UNDO*/
       gtk_widget_grab_focus (buttonList[NB_UNDO].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_UNDO].widget));
       return;
     case 'q':case 'Q':/*QUIT*/
       QuitApp (NULL, NULL);
       return;
     case 'h':case 'H':/*HELP*/
       gtk_widget_grab_focus (HelpButton);
       gtk_button_clicked (GTK_BUTTON (HelpButton));
       return;
     case 'f':case 'F':/*SELECT FONT*/
       gtk_menu_popup (GTK_MENU(GTK_OPTION_MENU (FontMenu)->menu),
                       NULL, NULL, NULL, NULL, 1, GDK_CURRENT_TIME);
       return;
     case 'd':case 'D':/*SELECT DOMAIN*/
       gtk_menu_popup (GTK_MENU(GTK_OPTION_MENU (ArgMenu)->menu),
                       NULL, NULL, NULL, NULL, 1, GDK_CURRENT_TIME);
       return;
     case GDK_Delete:
     case GDK_KP_Delete:/*ACL*/
       gtk_widget_grab_focus (buttonList[NB_ACL].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_ACL].widget));
       return;
     case GDK_BackSpace:/*CE/C*/
       gtk_widget_grab_focus (buttonList[NB_CE].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_CE].widget));
       return;
     case '-':/*+/-*/
       gtk_widget_grab_focus (buttonList[NB_PM].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_PM].widget));
       return;
     case 'a':case 'A':/*DRPI - the unit for displaying the argument*/
       gtk_widget_grab_focus (buttonList[NB_DRPI].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_DRPI].widget));
       return;
    }
    return;
  }
  
  else if (event->state & GDK_MOD1_MASK)/*If 'Alt' key is being pressed,*/
  {
    switch (kval)
    {   
     case 's':case 'S':
     case GDK_Return:
     case GDK_KP_Enter:/*STO*/
       gtk_widget_grab_focus (buttonList[NB_STO].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_STO].widget));
       return;
     case 'r':case 'R':/*RCL*/
       gtk_widget_grab_focus (buttonList[NB_RCL].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_RCL].widget));
       return;
     case '+':/*M+*/
       gtk_widget_grab_focus (buttonList[NB_MADD].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MADD].widget));
       return;
     case '-':/*M-*/
       gtk_widget_grab_focus (buttonList[NB_MSUB].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MSUB].widget));
       return;
     case '*':/*M**/
       gtk_widget_grab_focus (buttonList[NB_MMUL].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MMUL].widget));
       return;
     case '/':/*M/ */
       gtk_widget_grab_focus (buttonList[NB_MDIV].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MDIV].widget));
       return;
     case '^':/*M^*/
       gtk_widget_grab_focus (buttonList[NB_MPOW].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MPOW].widget));
       return;
     case 'c':case 'C':/*conjM*/
       gtk_widget_grab_focus (buttonList[NB_MCONJ].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_MCONJ].widget));
       return;
     case 'x':case 'X':/*QUIT*/
       QuitApp (NULL, NULL);
       return;
     case 'a':case 'A':/*HELP*/
       gtk_widget_grab_focus (HelpButton);
       gtk_button_clicked (GTK_BUTTON (HelpButton));
       return;
    }
    return;
  }
 
  else if (kval > 0x03d)  
  { 
    switch (kval)
    {
     case 'm':case 'M':
     case 's':case 'S':/*+/-*/
       gtk_widget_grab_focus (buttonList[NB_PM].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_PM].widget));
       return;
     case GDK_BackSpace:
       gtk_widget_grab_focus (buttonList[NB_BS].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_BS].widget));
       return;
     case 'e':case 'E':/*EE*/
       gtk_widget_grab_focus (buttonList[NB_EE].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_EE].widget));
       return;
     case GDK_Delete:
     case GDK_KP_Delete:/*CE/C*/
       gtk_widget_grab_focus (buttonList[NB_CE].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_CE].widget));
       return;
     case 'r':case 'R':/*real*/
       gtk_widget_grab_focus (buttonList[NB_REAL].widget);
       gtk_button_pressed (GTK_BUTTON (buttonList[NB_REAL].widget));
       gtk_button_released (GTK_BUTTON (buttonList[NB_REAL].widget));
       return;
     case 'i':case 'I':/*imag*/
       gtk_widget_grab_focus (buttonList[NB_IMAG].widget);
       gtk_button_pressed (GTK_BUTTON (buttonList[NB_IMAG].widget));
       gtk_button_released (GTK_BUTTON (buttonList[NB_IMAG].widget));
       return;
     case 'f':case 'F':/*abs*/
       gtk_widget_grab_focus (buttonList[NB_ABS].widget);
       gtk_button_pressed (GTK_BUTTON (buttonList[NB_ABS].widget));
       gtk_button_released (GTK_BUTTON (buttonList[NB_ABS].widget));
       return;
     case 'k':case 'K':/*arg*/
       gtk_widget_grab_focus (buttonList[NB_ARG].widget);
       gtk_button_pressed (GTK_BUTTON (buttonList[NB_ARG].widget));
       gtk_button_released (GTK_BUTTON (buttonList[NB_ARG].widget));
       return;
     case '^':/*z^w*/
       gtk_widget_grab_focus (buttonList[NB_POW].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_POW].widget));
       return;
     case 'c':case 'C':/*conj*/
       gtk_widget_grab_focus (buttonList[NB_CONJ].widget);
       gtk_button_clicked (GTK_BUTTON (buttonList[NB_CONJ].widget));
       return;
    }
    return;
  }
 else/*if digit or *, +, and so on.*/
  { 
    for (nIndex = NB_FIRST_SINGLE_LETTER_OP ; nIndex < nButtons ; nIndex++)
    {
      if ((char)kval == buttonList[nIndex].szLabel[0] &&
           buttonList[nIndex].szLabel[1] == '\0')
      {
        gtk_widget_grab_focus (buttonList[nIndex].widget);
        gtk_button_clicked (GTK_BUTTON (buttonList[nIndex].widget));
        return;
      }
    } 
  } 
}
