/*a simple calculator for complex number system
 *Akira Saitoh (1999,2005)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/

#define _GNU_SOURCE 1
#include <math.h>

#ifndef HAVE_ISINF
#ifdef isinf
#undef isinf
#endif /*isinf*/
#ifdef HAVE_IEEEFP_H
#include <ieeefp.h>
#define isinf(x) ((!finite((x)))*((x)<0.0?-1:1))
#else
#define isinf is_inf
static int is_inf (double x)
{
   if (x == HUGE_VAL) return (1);
   if (x == -HUGE_VAL) return (-1);
   return (0);
}
#endif /*HAVE_IEEEFP_H*/
#endif /*HAVE_ISINF*/

#include "OtherFuncs.h"
double factorial (int n)/*n(n-1)(n-2)(n-3)....*/
{
   int i;
   double S;

   S = (double)n; 
   if (n < 0) return 0;
   else if (n < 2) return 1;
   for (i = 1; i < n && (!isinf(S)); i++)
     S *= (double)i;
   return S;
}

double d_factorial (int n)/*n(n-2)(n-4)(n-6)....*/
{
   int i;
   double S;

   S = (double)n;
   if (n < -1) return 0;
   else if (n < 2) return 1;
   for (i = (--n); i > 2 && (!isinf(S)); i--)
      S *= (double)(--i);
   return S;
}

double nPr (int n, int r)
{
   int i,m;
   double S;

   S = (double)(m = n);
   if (n < 0 || (n < r || r < 0)) return 0;
   else if (r == 0) return 1;
   for (i = 1; i < r && (!isinf(S)); i++)
      S *= (double)(--m);
   return S;
}

double nCr (int n, int r)
{
   int i,m;
   double S;
   double U;

   S = (double)(m = n);
   if (n < 0 || (n < r || r < 0)) return 0;
   else if (n == r || r == 0) return 1;   
   if (n < (r+r)) r = (int)(n - r);
   U = (double)r;
   for (i = 1; i < r && (!isinf(S)); i++)
   {
      S *= (double)(--m);
      U *= (double)i;
   }
   return (S / U);
}

double nHr (int n, int r)
{
   int m;
   m = n + r - 1;
   return (nCr(m, r));
}
