/*cgtkcalc - a simple calculator for complex number system
 *Akira Saitoh (1999, 2002)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "RenewLabels.h"
#include "parenthesis.h"
#define S_SIZE 30
/*The precision specifies the number of siginificant digits on labels.*/
int l_precision = 8;

void RenewLabels()
{
   RenewRe();
   RenewIm();
   RenewAbs();
   RenewArg();
}
/*renew labelRe*/
void RenewRe()
{   
    gchar str[S_SIZE];
    g_snprintf (str, S_SIZE, "%.*G", l_precision, cz.r);
   if (strcmp (str, "-0") == 0)
   {
     strcpy (str, "0");
     cz.r = 0.0;
     /*When cz.abs == 0, users input argument firstly, I assume.*/
     if (cz.abs != 0.0)
       absarg(&cz);
   }
    gtk_label_set (GTK_LABEL (labelRe), str);
}
/*renew labelIm*/
void RenewIm()
{
    gchar str[S_SIZE];
    g_snprintf (str, S_SIZE, "%.*G", l_precision, cz.i);
   if (strcmp (str, "-0") == 0)
   {
     strcpy (str, "0");
     cz.i = 0.0;
     /*When cz.abs == 0, users input argument firstly, I assume.*/
     if (cz.abs != 0.0)
       absarg(&cz);
   }
    gtk_label_set (GTK_LABEL (labelIm), str);
}
/*renew labelAbs*/
void RenewAbs()
{ 
    gchar str[S_SIZE];
    g_snprintf (str, S_SIZE, "%.*G", l_precision, cz.abs);
    gtk_label_set (GTK_LABEL (labelAbs), str);
}
/*renew labelArg*/
void RenewArg()
{
    gchar str[S_SIZE];
    switch (arg_unit)
   {
   case RADIAN:/*rad*/
     g_snprintf (str, S_SIZE - 5, "%.*G", l_precision, cz.arg);
     if (strcmp(str, "-0") == 0) strcpy (str, "0");
     str_plus (str, " rad ");
     gtk_label_set (GTK_LABEL (labelArg), str);
     return;
   case PIRADIAN:/*PIrad*/
     /*translate rad into PIrad*/
     g_snprintf (str, S_SIZE - 5, "%.*G", l_precision, (cz.arg / tPI));
     if (strcmp(str, "-0") == 0) strcpy (str, "0");
     str_plus (str, "PIrad");
     gtk_label_set (GTK_LABEL (labelArg), str);
     return;
   case DEGREE:/*deg*/
     /*translate rad into deg*/
     g_snprintf (str, S_SIZE - 5, "%.*G", l_precision, (180.0 / tPI * cz.arg));
     if (strcmp(str, "-0") == 0) strcpy (str, "0");
     str_plus (str, " deg ");
     gtk_label_set (GTK_LABEL (labelArg), str);
     return;
   }
}

/*Display memorized value.*/      
void RenewMem()
{
   char *str = NULL;
   char strMR[S_SIZE];
   char strMI[S_SIZE];
   char strpm[4] = " + ";

   g_snprintf (strMR, S_SIZE, "%.*G", l_precision, mz.r);
   if (strcmp (strMR, "-0") == 0) strcpy (strMR, "0");
   if ((float)mz.i < 0.0)
   {
      g_snprintf (strMI, S_SIZE, "%.*G", l_precision, -mz.i);
      strcpy (strpm, " - ");
   }
   else
   {
      g_snprintf (strMI, S_SIZE, "%.*G", l_precision, mz.i);
   }
   if (strcmp (strMI, "-0") == 0) strcpy (strMI, "0");

   gtk_label_set (GTK_LABEL (labelMem),
                  str = g_strconcat ("M = ", strMR, strpm,
                                     strMI, " i", NULL));
   g_free (str);
}
/*Display current operator.*/
void RenewOp()
{
   /*Show as many left parentheses as used.*/
   if (p_count)
   {
      int i;
      char *buf = NULL;
      buf = (char *) malloc ((N_P_B + CMD_SIZE + 1) * sizeof(char));
      if (buf == NULL)
      {
         gtk_label_set (GTK_LABEL (labelOp), prevCmd);
         return;
      }
      for (i = 0; i < p_count; i++)
         *(buf + i) = '(';
      *(buf + i) = ' ';
      *(buf + (++i)) = '\0';
      if (*prevCmd != '(' && *prevCmd != ')')
         str_plus (buf, prevCmd);
      gtk_label_set (GTK_LABEL (labelOp), buf);
      free (buf);
   }
   else gtk_label_set (GTK_LABEL (labelOp), prevCmd);

}

/*called when Back-space is pressed*/
void BSFunc()
{  
   char *ltext, str0[BUF_SIZE], str1[BUF_SIZE];
   if (inputpos == REAL)
   {
     gtk_label_get (GTK_LABEL (labelRe), &ltext);
     strncpy_terminate (str0, ltext, BUF_SIZE);
     g_strchug (str0);
     if (strlen(str0) < 2)  strcpy (str0, "0");
     else{
       strncpy_terminate (str1, str0, strlen(str0) - 1);
       strcpy (str0, str1);
     }
     if(str0[strlen(str0) - 1] == 'E')
        strncpy_terminate (str0, str1, strlen(str1) - 1);
     cz.r = atof (str0);
     absarg (&cz);
     RenewAbs();RenewArg();
     gtk_label_set (GTK_LABEL (labelRe), str0);
     return;
   }
   else if (inputpos == IMAG)
   {
     gtk_label_get (GTK_LABEL (labelIm), &ltext);
     strncpy_terminate (str0, ltext, BUF_SIZE);
     g_strchug (str0);
     if (strlen(str0) < 2)  strcpy (str0, "0");
     else{
       strncpy_terminate (str1, str0, strlen(str0) - 1);
       strcpy (str0, str1);
     }
     if(str0[strlen(str0) - 1] == 'E')
        strncpy_terminate (str0, str1, strlen(str1) - 1);
     cz.i = atof (str0);
     absarg (&cz);
     RenewAbs();RenewArg();
     gtk_label_set (GTK_LABEL (labelIm), str0);
     return;
   }
   else if (inputpos == ABSO)
   {
     gtk_label_get (GTK_LABEL (labelAbs), &ltext);
     strncpy_terminate (str0, ltext, BUF_SIZE);
     g_strchug (str0);
     if (strlen(str0) < 2)  strcpy (str0, "0");
     else{
       strncpy_terminate (str1, str0, strlen(str0) - 1);
       strcpy (str0, str1);
     }
     if(str0[strlen(str0) - 1] == 'E')
        strncpy_terminate (str0, str1, strlen(str1) - 1);
     cz.abs = atof (str0);
     realimag (&cz);
     RenewRe();RenewIm();
     gtk_label_set (GTK_LABEL (labelAbs), str0);
     return;
   }       
   else
   { 
      gtk_label_get (GTK_LABEL (labelArg), &ltext);
      strncpy_terminate (str0, ltext, BUF_SIZE);
      g_strchug (str0);
      if (strlen(str0) < 7)
      /*Because `str' includes " rad " or " deg ", it must be trimed. */
      /*And one more character should be trimed.*/
      {
         strcpy (str0, "0");
      }
      else
      {
         strncpy_terminate (str1, str0, strlen(str0) - 6);/* 6 = 1 + length of " rad " */
         strcpy (str0, str1);
      }

       if(str0[strlen(str0) - 1] == 'E')
          strncpy_terminate (str0, str1, strlen(str1) - 1);
             
      switch (arg_unit){
        case RADIAN:cz.arg = atof (str0); break;
      /*translate PIrad into rad*/
        case PIRADIAN:cz.arg = tPI * atof (str0); break;
      /*translate deg into rad*/
        case DEGREE:cz.arg = (tPI / 180.0 * atof (str0)); break;
      }
      realimag (&cz);
      RenewRe();RenewIm();
      
      switch (arg_unit){
        case RADIAN: str_plus (str0, " rad ");break;
        case PIRADIAN: str_plus (str0, "PIrad");break;
        case DEGREE: str_plus (str0, " deg ");break;
      }
      gtk_label_set (GTK_LABEL (labelArg), str0);
      return;
   }   
}

/*rotate stack before cz is changed*/
void PrepareChange()
{ 
   int i;
   char *str;
   for (i = N_MI - 1; i > 0; i--)
   {
      buf_state[i].cz = buf_state[i-1].cz;
      buf_state[i].mz = buf_state[i-1].mz;
      buf_state[i].is_hidden_m = buf_state[i-1].is_hidden_m;
      buf_state[i].p_count = buf_state[i-1].p_count;
      copy_inputs (buf_state[i].inputs, buf_state[i-1].inputs);
      strncpy (buf_state[i].prevCmd, buf_state[i-1].prevCmd, CMD_SIZE);
   }
   buf_state[0].cz = cz;
   buf_state[0].mz = mz;
   gtk_label_get (GTK_LABEL (labelMem), &str);
   if (*str == '\0')  buf_state[0].is_hidden_m = 1;
   else  buf_state[0].is_hidden_m = 0;
   buf_state[0].p_count = p_count;
   strncpy (buf_state[0].prevCmd, prevCmd, CMD_SIZE);
   copy_inputs (buf_state[0].inputs, bufinputs);

   if (n_memInputs != N_MI) n_memInputs++;
}
