/*a simple calculator for complex number system
 *Akira Saitoh (1999)
 *Most of this file originated with Eric Harlow.
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "auxiliary.h"

void TrimTrailingZeros (char *szDigits)
{
   int nIndex;
   int bDecimal = FALSE;
   int nPos = -1;
   for (nIndex = 0; nIndex < (int)strlen(szDigits); nIndex++)
   {
      if (szDigits[nIndex] == '.')
        bDecimal = TRUE;
      if (bDecimal)
      {
         if (szDigits[nIndex] == '0')
         {
           if(nPos < 0)
             nPos = nIndex;
           else
             nPos = -1;
         }
      }
   }   
   if (nPos > 0)
     szDigits[nPos] = '\0';
}

void TrimLeadingZeros (char *szDigits)
{
   int nPos;
   if (*szDigits == '\0')
     return;
   for (nPos = 0; (szDigits[nPos] && szDigits[nPos] == '0'); nPos++)
   {
     if (isdigit (szDigits[nPos + 1]))
       szDigits[nPos] = ' ';
   }
   if (*szDigits == ' ') g_strchug (szDigits);
}

void TrimLeadingSigns (char *str)
{
   int nPos;
   int nSigns = 1;
   if (*str == '\0')
     return;
   g_strchomp (str);
   if (*str == ' ') g_strchug (str);
   for (nPos = 0; (str[nPos] && str[nPos] == '-'); nPos++)
   {
     if (str[nPos + 1] == '-')
     {
       str[nPos] = ' ';
       nSigns++;
     }
   }
   if (nSigns % 2 == 0)
     str[nPos - 1] = ' ';
   if (*str == ' ') g_strchug (str);
   if (*str == '\0') strcpy (str, "0");
}

int is_Command (char *str)
{
  char ch = *str;
  switch (ch)
   {
      case '/':
      case '*':
      case '+':
      case '-':
      case '=':
      case '(':
      case ')':
        return (TRUE);
   }
   if (strncmp (str,"mod",3) == 0)
     return (TRUE);
   else if (strncmp (str,"z^w",3) == 0)
     return (TRUE);
   else if (strncmp (str,"nHr",3) == 0)
     return (TRUE);
   else if (strncmp (str,"nPr",3) == 0)
     return (TRUE);
   else if (strncmp (str,"nCr",3) == 0)
     return (TRUE);
   else if (strncmp (str,"Hn(z)",5) == 0)
     return (TRUE);
   else if (strncmp (str,"Hn*(z)",6) == 0)
     return (TRUE);
   else if (strncmp (str,"u_op",4) == 0)
     return (TRUE);

   return (FALSE);
}

int is_int (double r)
{
   double f, c;
   f = floor (r);
   c = ceil (r);
   if (r < f + 1.0e-5 && r < f + abs(f) * 1.0e-14)
      return 1;
   if (c - 1.0e-5 < r && c - abs(c) * 1.0e-14 < r)
      return 1;
   if (r < f + 1.0e-19 || c - 1.0e-19 < r)
      return 1;
   return 0;
}

/*Yet Another round function*/
double yaround (double r)
{
   double f, c;
   f = floor (r);
   c = ceil (r);
   if (r - f < c - r)
      return f;
   else
      return c;
}

/*add a string to sufficiently large buffer str.*/
char *str_plus (gchar *str, gchar *plus)
{
   char *ss;
   ss = g_strconcat (str, plus, NULL);
   strcpy (str, ss);
   g_free (ss);
   return (str);
}

/*add a string to sufficiently large buffer str.*/
char *plus_str (gchar *plus, gchar *str)
{
   char *ss;
   ss = g_strconcat (plus, str, NULL);
   strcpy (str, ss);
   g_free (ss);
   return (str);
}

/*do ordinary strcpy and terminate 'dest'.*/
char *strncpy_terminate (char *dest, char *src, int n)
{
   strncpy (dest, src, n);
   /*We must terminate 'dest' here.*/
   dest[n] = '\0';
   return (dest);
}

/*return where c is in str.*/
int where_c_in_str (char *str, char c)
{
   int i,l;
   l = strlen (str);
   for (i = 0; i < l; i++)
   {
      if (*(str + i) == c)
         return (i);
   }
   return (0);
}

/*Clear inputs. (typInputs is declared in "cgtkcalc.h".)*/
void clear_inputs (typInputs inputs[N_PR])
{
   int i;
   for (i = 0; i < N_PR; i++)
   {
      inputs[i].cmd[0] = '\0';
      inputs[i].pr = 0;
   }
}

/*Copy inputs. (typInputs is declared in "cgtkcalc.h".)*/
void copy_inputs (typInputs d_inputs[N_PR], typInputs s_inputs[N_PR])
{
   int i;
   for (i = 0; i < N_PR; i++)
   {
      d_inputs[i].z = s_inputs[i].z;
      strncpy (d_inputs[i].cmd, s_inputs[i].cmd, CMD_SIZE);
      d_inputs[i].pr = s_inputs[i].pr;
   }
}
