/*a simple calculator for complex number system
 *Akira Saitoh (1999-2017)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 * [3] Hiroyuki Tanaka: GTK+ nyuumon (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include <stdlib.h>
#include "DEF_VERSION.h"
#include "cgtkcalc.h"
#include "Copy.h"
#include "CreateButtons.h"
#include "CreateHbox.h"
#include "CreateHelpWindow.h"
#include "CreateLabels.h"
#include "KeyPressed.h"
#include "RenewLabels.h"
#include "CmdlineOptions.h"
#include "WindowIcon.h"

#define _HELP_STR_0 \
"cgtkcalc  "
#define _HELP_STR_1 \
"Usage: cgtkcalc [options]\n\
Options:\n\
 -h, --help                  Display this message.\n\
 -s, --shrunk, --simple      Shrunk mode.\n\
 -i, --hide                  Hide abs. and arg.\n\
 -w, --record                Write a record to stdout.\n\
 -H, --Hand                  Use a hand cursor.\n\
 -p, --PIrad (-d, --deg)     Use PIrad (deg) as the unit for expressing\
 the argument.\n\
 -D, --2pi                   Set the domain of the argument to [0, 2PI).\n\
 -prec, --prec <number>      Specify the precision.\
 It must be in {1,2,...,10,11}; 8 by default.\n\n\
 -rc, --rc <rc file>         Specify the gtk+ rc file.\n\n\
 -f, --font [family] [big]   Specify a font-family name\
 (`century\', `fixed\', `helvetica\' or `times\'); `fixed\' by default.\
 Specify `big\' to use a big size of font.\n\n\
 -fc-real (-fc-abs, -fc-op) <colorname>    Specify the foreground color for\
 real and imag. parts (for abs. and arg.,\
 for operator and mem. displays)."

/*Font color for labelRe, labelAbs, labelOp.*/
GdkColor fc_real = {0,0,0,0}, fc_abs = {0,0,0,0}, fc_op = {0,0,0,0};
/*main window*/
static GtkWidget *mwindow;

static void SetStyle (int stv);/*`stv' is a variable for setting some fonts.*/
static gint CloseAppWindow (GtkWidget *widget, gpointer data);


static gint CloseAppWindow (GtkWidget *widget, gpointer data)
{
   gtk_widget_destroy (GTK_WIDGET (mwindow));
   if (CopiedStr) free (CopiedStr);
   free_helpbox_str ();
   gtk_main_quit ();
   return (FALSE);
}

void QuitApp (GtkWidget *widget, gpointer data)
{
   gtk_widget_destroy (GTK_WIDGET (mwindow));
   if (CopiedStr) free (CopiedStr);
   free_helpbox_str ();
   gtk_main_quit ();
   return;
}
                      
/*With this style, the labels on the window imitate LCD.
 *`stv' is a variable for setting fonts.*/
GtkStyle *CreateStyle (int stv)
{ 
   GtkStyle *defstyle;
   GtkStyle *style;
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   PangoFontDescription *font_desc = NULL;
#else /*GTK+-1.2*/
   GdkFont *fixed_font = NULL;
#endif /*GTK+-X.X*/
   GdkColor color_light = {0, 0xffff, 0xffff, 0xc000};
   GdkColor color_lightgray = {0, 0xf300, 0xea00, 0xe000};
   GdkColor color_lightgreen = {0, 0xc000, 0xffff, 0xa500};
   GdkColor color_ironblack = {0, 0x1900, 0x2800, 0x1900};

   gdk_color_alloc (gdk_colormap_get_system (), &color_light);
   gdk_color_alloc (gdk_colormap_get_system (), &color_ironblack);

   defstyle = gtk_widget_get_default_style ();
   style = gtk_style_copy (defstyle);
   /*Here, M_ABS or M_OP is a kind of mask to set another color in the style.*/
   /*Font color for labelRe and labelIm.*/
   if (stv < M_ABS)
   {
      style->fg[GTK_STATE_NORMAL] = (fc_real.red || fc_real.green || fc_real.blue ?
                                     fc_real : color_light);
      style->text[GTK_STATE_NORMAL] = style->fg[GTK_STATE_NORMAL];
   }
   /*Font color for labelAbs and labelArg.*/
   else if (stv < M_OP)
   {
      style->fg[GTK_STATE_NORMAL] = (fc_abs.red || fc_abs.green || fc_abs.blue ?
                                     fc_abs : color_lightgreen);
      style->text[GTK_STATE_NORMAL] = style->fg[GTK_STATE_NORMAL];
      stv ^= M_ABS;/*take off mask*/
   }
   /*Font color for labelOp and labelMem.*/
   else
   {
      style->fg[GTK_STATE_NORMAL] = (fc_op.red || fc_op.green || fc_op.blue ?
                                     fc_op : color_lightgray);
      style->text[GTK_STATE_NORMAL] = style->fg[GTK_STATE_NORMAL];
      stv ^= M_OP;/*take off mask*/
      stv -= 4;/*Reduce stv to use a font of smaller size.*/
   }
   style->bg[GTK_STATE_NORMAL] = color_ironblack;

   switch (stv)
   {
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
    case F_FIXED:
     font_desc = pango_font_description_from_string ("fixed bold 13");
     break;
    case F_CENTURY:
     font_desc = pango_font_description_from_string ("new century schoolbook bold 14");
     break;
    case F_HELVETICA:
     font_desc = pango_font_description_from_string ("helvetica bold 14");
     break;
    case F_TIMES:
     font_desc = pango_font_description_from_string ("times bold 14");
     break;
    case BIG_FIXED:
     font_desc = pango_font_description_from_string ("fixed bold 16");
     break;
    case BIG_CENTURY:
     font_desc = pango_font_description_from_string ("new century schoolbook bold 16");
     break;
    case BIG_HELVETICA:
     font_desc = pango_font_description_from_string ("helvetica bold 16");
     break;
    case BIG_TIMES:
     font_desc = pango_font_description_from_string ("times bold 16");
     break;
#else /*GTK+-1.2*/
    case F_FIXED:
     fixed_font = gdk_font_load ("-*-fixed-bold-r-*-*-*-190-*-*-*-*-iso8859-1");
     break;
    case F_CENTURY:
     fixed_font = gdk_font_load ("-*-new century schoolbook-bold-r-*-*-*-180-*-*-*-*-iso8859-1");
     break;
    case F_HELVETICA:
     fixed_font = gdk_font_load ("-*-helvetica-bold-r-*-*-*-180-*-*-*-*-iso8859-1");
     break;
    case F_TIMES:
     fixed_font = gdk_font_load ("-*-times-bold-r-*-*-*-180-*-*-*-*-iso8859-1");
     break;
    case BIG_FIXED:
     fixed_font = gdk_font_load ("-*-fixed-bold-r-*-*-*-230-*-*-*-*-iso8859-1");
     break;
    case BIG_CENTURY:
     fixed_font = gdk_font_load ("-*-new century schoolbook-bold-r-*-*-*-240-*-*-*-*-iso8859-1");
     break;
    case BIG_HELVETICA:
     fixed_font = gdk_font_load ("-*-helvetica-bold-r-*-*-*-240-*-*-*-*-iso8859-1");
     break;
    case BIG_TIMES:
     fixed_font = gdk_font_load ("-*-times-bold-r-*-*-*-240-*-*-*-*-iso8859-1");
     break;
#endif /*GTK+-X.X*/
   }

#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   if (font_desc && stv > -1)
      style->font_desc = font_desc;
#else /*GTK+-1.2*/
   if (fixed_font && stv > -1)
      style->font = fixed_font;
#endif /*GTK+-X.X*/

   return (style);
}

/*`stv' is a variable for setting some fonts.*/
static void SetStyle (int stv)
{  
   GtkStyle *style0;

   style0 = CreateStyle (0);
   gtk_widget_set_style (mwindow, style0);
   gtk_widget_set_style (event_boxRe, style0);
   gtk_widget_set_style (event_boxIm, style0);
   gtk_widget_set_style (event_boxAbs, style0);
   gtk_widget_set_style (event_boxArg, style0);
   gtk_style_unref (style0);
   /*The style for labels should be independent of that for the windows.*/
   style0 = CreateStyle (stv);
   gtk_widget_set_style (labelRe, style0);
   gtk_widget_set_style (labelIm, style0);
   gtk_style_unref (style0);

   style0 = CreateStyle (stv ^ M_ABS);
   gtk_widget_set_style (labelAbs, style0);
   gtk_widget_set_style (labelArg, style0);
   gtk_style_unref (style0);

   style0 = CreateStyle (stv ^ M_OP);
   gtk_widget_set_style (labelMem, style0);
   gtk_widget_set_style (labelOp, style0);
   gtk_style_unref (style0);
}

int main (int argc, char *argv[])
{
   int stv = 0;/*`stv' is a variable for setting some fonts.*/
   GtkWidget *vbox;
   GtkWidget *table;
   GtkWidget *hbox;

   /*Do nothing but exit after help.*/
   if (argc == 2)
      if (strcmp (argv[1], "--help") == 0 ||
          strcmp (argv[1], "-h") == 0)
       {
         printf ("%s", _HELP_STR_0);
         puts (_VERSION_STR);
         puts (_HELP_STR_1);
         return (0);
       }

   /*I think locale does not have to be set in this application.
    *Locale is set only when $LANG is "ja_JP.eucJP" or $LANGUAGE is.*/
#ifdef  HAVE_LIBGLIB_2_0 /*GTK+-2.0*/
   gtk_disable_setlocale ();
#endif /*GTK+-2.0*/
   {
      char *lstr = NULL;
      if ((lstr = getenv ("LANG")))
      {
         if (strcmp ((const char*) lstr, "ja_JP.eucJP") == 0)
         {
            gtk_set_locale ();
         }
         else if ((lstr = getenv ("LANGUAGE")))
         {
            if (strcmp ((const char*) lstr, "ja_JP.eucJP") == 0)
               gtk_set_locale ();
         }
      }
   }

   gtk_init (&argc, &argv);

   /*Create main window and many widgets on it.*/

   mwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (mwindow), "cgtkcalc");
   setup_window_icon (GTK_WINDOW (mwindow));/*defined in "WindowIcon.c"*/
   gtk_window_set_default_size (GTK_WINDOW (mwindow), 340, 370);
   gtk_signal_connect (GTK_OBJECT (mwindow), "key_press_event",
                      GTK_SIGNAL_FUNC (key_press), NULL);
   gtk_signal_connect (GTK_OBJECT (mwindow), "delete_event",
                      GTK_SIGNAL_FUNC (CloseAppWindow), NULL);

   vbox = gtk_vbox_new (FALSE,0);
   gtk_container_add (GTK_CONTAINER (mwindow), vbox);
   gtk_widget_show (vbox);
  
   hbox = CreateHbox ();
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show (hbox);
  
   table = gtk_table_new (12, 8, TRUE);
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
   gtk_widget_show (table);

   CreateCalculatorButtons (table);/*defined in "CreateButtons.c"*/
    
   CreateLabels (table);/*defined in "CreateLabels.c"*/

   /*We must realize table to set another cursor.*/
   gtk_widget_realize (table);

   /*Command-line options.
    *`stv' is a variable for setting fonts.*/
   handle_command_line_options (argc, argv, mwindow, table, hbox, &stv);

   setup_selection ();/*defined in "Copy.c"*/
   SetStyle (stv);

   /*Toggle `real' button.*/ 
   gtk_button_released (GTK_BUTTON (buttonList[NB_REAL].widget));
   /*Show initial values on labels.*/
   RenewLabels ();

   gtk_widget_show (mwindow);

   gtk_main ();
  
   return (0);
}
