/*a simple calculator for complex number system
 *Akira Saitoh (2001)
 *---------------------------------------------------------
 *References
 * [1] Eric Harlow: DEVELOPING LINUX APPLICATIONS by Eric Harlow (1999)
 * [2] Eiji Takeda: GTK+ de hajimeru X programming (1999)
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif/*HAVE_CONFIG_H*/
#include "cgtkcalc.h"
#include "auxiliary.h"
#include "parenthesis.h"

int p_count = 0;
/*Buffer for values used with parentheses.*/
/*And buffer for commands used with parentheses.*/
static p_buffer p_buf[N_P_B];/*must be allocated as many as N_P_B.*/

static void shift_p_buf ();
static void r_shift_p_buf ();
/*Handle '('.*/
void handle_left_parenthesis (typInputs inputs[N_PR])
{
   if ((++p_count) == N_P_B)
   {
      p_count = 0;
      return;
   }
   /*Push previous state into the stack.*/
   shift_p_buf ();
   copy_inputs (p_buf[0].inputs, inputs);
   /*Clear current state to prepare for inputs in the parentheses.*/
   /*This function is defined in "auxiliary.c".*/
   clear_inputs (inputs);
}
/*Handle ')'.*/
void handle_right_parenthesis (typInputs inputs[N_PR])
{
   /*We have only to do in the reverse order as left parenthesis.*/
   /*Pop previous state out the stack.*/
   copy_inputs (inputs, p_buf[0].inputs);
   r_shift_p_buf ();
   p_count--;
   /*Backup for UNDO*/
   copy_inputs (p_buf[N_P_B - 1].inputs, inputs);
}

static void shift_p_buf ()
{
   int i;
   for (i = p_count - 1; i > 0; i--)
   {
      copy_inputs (p_buf[i].inputs, p_buf[i-1].inputs);
   }
}
/*Reverse-shift for p_buf.*/
static void r_shift_p_buf ()
{
   int i;
   for (i = 0; i < p_count - 1; i++)
   {
      copy_inputs (p_buf[i].inputs, p_buf[i+1].inputs);
   }
}

/* Function to restore the state of parentheses for UNDO. */
void undo_parentheses (char c)
{
   switch (c)
   {
      case '(':
         r_shift_p_buf ();
         p_count--;
         break;
      case ')':
      {
         typInputs buf[N_PR];
         copy_inputs (buf, p_buf[N_P_B - 1].inputs);
         p_count++;
         shift_p_buf ();
         copy_inputs (p_buf[0].inputs, buf);
      }  break;
   }
}
