/* zcomplex.h
   Author: Akira Saitoh (1997,2002) */
/*-------------------------------------------------------------------*/
/* When you compile "zcomplex.c", please compile it with -lm         */
/* For example, gcc zcomplex.c -lm                                   */
/*-------------------------------------------------------------------*/
/* NO WARRANTY OF ANYKIND. This program is free software.            */
/*-------------------------------------------------------------------*/
/*      real part : z.r, imaginary part : z.i                        */
/* absolute value : z.abs,     argument : z.arg                      */
/*-------------------------------------------------------------------*/

#ifndef __ZCOMPLEX_H__
#define __ZCOMPLEX_H__

typedef struct
{
  double r;
  double i;
  double abs;
  double arg;
} zcomplex;
/*
 *The argument value is between -PI and PI by default.
 *If it should be between 0 and 2PI, bt0_2PI should be 1.
 */
extern int bt0_2PI;
/*
 *The maximum for the value 'n' of Hermite Polynomials.
 */
/*
extern int max_n_of_hermite;
*/
/**/
/*Calculate the absolute value and the argument of the number
 *from its real part and imaginary part.*/
void absarg(zcomplex *a);
/*Calculate real part and imaginary part of the number
 *from its absolute value and argument.*/
void realimag(zcomplex *a);
/**/
/*Create a complex number from real part and imaginary part.*/
/*
zcomplex zcreate(double real, double imag);
*/
/*Create a complex number from the absolute value and the argument.*/
/*
zcomplex zcreate2(double abs, double arg);
*/
/**/
zcomplex zconj(zcomplex a);
zcomplex zcm(zcomplex a);/*"-a"*/
zcomplex zcadd(zcomplex a, zcomplex b);
zcomplex zcsub(zcomplex a, zcomplex b);
zcomplex zcmul(zcomplex a, zcomplex b);
zcomplex zcdiv(zcomplex a, zcomplex b);
zcomplex zcpow(zcomplex a, zcomplex b);
/**/
zcomplex zcfmod(zcomplex a, zcomplex b);
/**/
zcomplex zclog(zcomplex a);
zcomplex zcexp(zcomplex a);
/**/
zcomplex zcsin(zcomplex a);   zcomplex zccos(zcomplex a);   zcomplex zctan(zcomplex a);
zcomplex zcasin(zcomplex a);  zcomplex zcacos(zcomplex a);  zcomplex zcatan(zcomplex a);
zcomplex zcsinh(zcomplex a);  zcomplex zccosh(zcomplex a);  zcomplex zctanh(zcomplex a);
zcomplex zcasinh(zcomplex a); zcomplex zcacosh(zcomplex a); zcomplex zcatanh(zcomplex a);
/**/
zcomplex zcgamma(zcomplex a);
/**/
/*Hermite polynomial*/
/*[1].Hn'(z)=nHn-1(z), weight w(x)=exp[(-x^2)/2]*/
/*
zcomplex Hn(int n, zcomplex a);
*/
/*[2].Hn'(z)=2nHn-1(z), weight w(x)=exp[-x^2]*/
/*
zcomplex Hn_s(int n, zcomplex a);
*/
#endif/*__ZCOMPLEX_H__*/
